# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.custom_attribute_definition import CustomAttributeDefinitionParams
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.create_order_custom_attribute_definition_response import CreateOrderCustomAttributeDefinitionResponse
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.retrieve_order_custom_attribute_definition_response import RetrieveOrderCustomAttributeDefinitionResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...types.update_order_custom_attribute_definition_response import UpdateOrderCustomAttributeDefinitionResponse
from ...types.delete_order_custom_attribute_definition_response import DeleteOrderCustomAttributeDefinitionResponse
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCustomAttributeDefinitionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateOrderCustomAttributeDefinitionResponse]:
        """
        Creates an order-related custom attribute definition.  Use this endpoint to
        define a custom attribute that can be associated with orders.

        After creating a custom attribute definition, you can set the custom attribute for orders
        in the Square seller account.

        Parameters
        ----------
        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition to create. Note the following:
            - With the exception of the `Selection` data type, the `schema` is specified as a simple URL to the JSON schema
            definition hosted on the Square CDN. For more information, including supported values and constraints, see
            [Specifying the schema](https://developer.squareup.com/docs/customer-custom-attributes-api/custom-attribute-definitions#specify-schema).
            - If provided, `name` must be unique (case-sensitive) across all visible customer-related custom attribute definitions for the seller.
            - All custom attributes are visible in exported customer data, including those set to `VISIBILITY_HIDDEN`.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency.
            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateOrderCustomAttributeDefinitionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/orders/custom-attribute-definitions",
            method="POST",
            json={
                "custom_attribute_definition": convert_and_respect_annotation_metadata(
                    object_=custom_attribute_definition, annotation=CustomAttributeDefinitionParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateOrderCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=CreateOrderCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, key: str, *, version: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RetrieveOrderCustomAttributeDefinitionResponse]:
        """
        Retrieves an order-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.

        To retrieve a custom attribute definition created by another application, the `visibility`
        setting must be `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to retrieve.

        version : typing.Optional[int]
            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include this optional field and specify the current version of the custom attribute.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RetrieveOrderCustomAttributeDefinitionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/orders/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="GET",
            params={
                "version": version,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveOrderCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=RetrieveOrderCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        key: str,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateOrderCustomAttributeDefinitionResponse]:
        """
        Updates an order-related custom attribute definition for a Square seller account.

        Only the definition owner can update a custom attribute definition. Note that sellers can view all custom attributes in exported customer data, including those set to `VISIBILITY_HIDDEN`.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to update.

        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition that contains the fields to update. This endpoint supports sparse updates,
            so only new or changed fields need to be included in the request.  For more information, see
            [Updatable definition fields](https://developer.squareup.com/docs/orders-custom-attributes-api/custom-attribute-definitions#updatable-definition-fields).

            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency) control, include the optional `version` field and specify the current version of the custom attribute definition.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency.
            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateOrderCustomAttributeDefinitionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/orders/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="PUT",
            json={
                "custom_attribute_definition": convert_and_respect_annotation_metadata(
                    object_=custom_attribute_definition, annotation=CustomAttributeDefinitionParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateOrderCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=UpdateOrderCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteOrderCustomAttributeDefinitionResponse]:
        """
        Deletes an order-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.

        Only the definition owner can delete a custom attribute definition.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteOrderCustomAttributeDefinitionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/orders/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteOrderCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=DeleteOrderCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawCustomAttributeDefinitionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateOrderCustomAttributeDefinitionResponse]:
        """
        Creates an order-related custom attribute definition.  Use this endpoint to
        define a custom attribute that can be associated with orders.

        After creating a custom attribute definition, you can set the custom attribute for orders
        in the Square seller account.

        Parameters
        ----------
        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition to create. Note the following:
            - With the exception of the `Selection` data type, the `schema` is specified as a simple URL to the JSON schema
            definition hosted on the Square CDN. For more information, including supported values and constraints, see
            [Specifying the schema](https://developer.squareup.com/docs/customer-custom-attributes-api/custom-attribute-definitions#specify-schema).
            - If provided, `name` must be unique (case-sensitive) across all visible customer-related custom attribute definitions for the seller.
            - All custom attributes are visible in exported customer data, including those set to `VISIBILITY_HIDDEN`.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency.
            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateOrderCustomAttributeDefinitionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/orders/custom-attribute-definitions",
            method="POST",
            json={
                "custom_attribute_definition": convert_and_respect_annotation_metadata(
                    object_=custom_attribute_definition, annotation=CustomAttributeDefinitionParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateOrderCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=CreateOrderCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, key: str, *, version: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RetrieveOrderCustomAttributeDefinitionResponse]:
        """
        Retrieves an order-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.

        To retrieve a custom attribute definition created by another application, the `visibility`
        setting must be `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to retrieve.

        version : typing.Optional[int]
            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include this optional field and specify the current version of the custom attribute.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RetrieveOrderCustomAttributeDefinitionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/orders/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="GET",
            params={
                "version": version,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveOrderCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=RetrieveOrderCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        key: str,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateOrderCustomAttributeDefinitionResponse]:
        """
        Updates an order-related custom attribute definition for a Square seller account.

        Only the definition owner can update a custom attribute definition. Note that sellers can view all custom attributes in exported customer data, including those set to `VISIBILITY_HIDDEN`.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to update.

        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition that contains the fields to update. This endpoint supports sparse updates,
            so only new or changed fields need to be included in the request.  For more information, see
            [Updatable definition fields](https://developer.squareup.com/docs/orders-custom-attributes-api/custom-attribute-definitions#updatable-definition-fields).

            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency) control, include the optional `version` field and specify the current version of the custom attribute definition.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency.
            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateOrderCustomAttributeDefinitionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/orders/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="PUT",
            json={
                "custom_attribute_definition": convert_and_respect_annotation_metadata(
                    object_=custom_attribute_definition, annotation=CustomAttributeDefinitionParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateOrderCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=UpdateOrderCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteOrderCustomAttributeDefinitionResponse]:
        """
        Deletes an order-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.

        Only the definition owner can delete a custom attribute definition.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteOrderCustomAttributeDefinitionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/orders/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteOrderCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=DeleteOrderCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .catalog_quick_amount_type import CatalogQuickAmountType
import pydantic
from .money import Money
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogQuickAmount(UncheckedBaseModel):
    """
    Represents a Quick Amount in the Catalog.
    """

    type: CatalogQuickAmountType = pydantic.Field()
    """
    Represents the type of the Quick Amount.
    See [CatalogQuickAmountType](#type-catalogquickamounttype) for possible values
    """

    amount: Money = pydantic.Field()
    """
    Represents the actual amount of the Quick Amount with Money type.
    """

    score: typing.Optional[int] = pydantic.Field(default=None)
    """
    Describes the ranking of the Quick Amount provided by machine learning model, in the range [0, 100].
    MANUAL type amount will always have score = 100.
    """

    ordinal: typing.Optional[int] = pydantic.Field(default=None)
    """
    The order in which this Quick Amount should be displayed.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .customer_text_filter import CustomerTextFilter
import pydantic
from .country import Country
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CustomerAddressFilter(UncheckedBaseModel):
    """
    The customer address filter. This filter is used in a [CustomerCustomAttributeFilterValue](entity:CustomerCustomAttributeFilterValue) filter when
    searching by an `Address`-type custom attribute.
    """

    postal_code: typing.Optional[CustomerTextFilter] = pydantic.Field(default=None)
    """
    The postal code to search for. Only an `exact` match is supported.
    """

    country: typing.Optional[Country] = pydantic.Field(default=None)
    """
    The country code to search for.
    See [Country](#type-country) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .timecard import TimecardParams
from .error import ErrorParams


class SearchTimecardsResponseParams(typing_extensions.TypedDict):
    """
    The response to a request for `Timecard` objects. The response contains
    the requested `Timecard` objects and might contain a set of `Error` objects if
    the request resulted in errors.
    """

    timecards: typing_extensions.NotRequired[typing.Sequence[TimecardParams]]
    """
    Timecards.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    An opaque cursor for fetching the next page.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .terminal_refund_query_filter import TerminalRefundQueryFilter
import pydantic
from .terminal_refund_query_sort import TerminalRefundQuerySort
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TerminalRefundQuery(UncheckedBaseModel):
    filter: typing.Optional[TerminalRefundQueryFilter] = pydantic.Field(default=None)
    """
    The filter for the Terminal refund query.
    """

    sort: typing.Optional[TerminalRefundQuerySort] = pydantic.Field(default=None)
    """
    The sort order for the Terminal refund query.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawAccountsClient
from ...requests.loyalty_account import LoyaltyAccountParams
from ...core.request_options import RequestOptions
from ...types.create_loyalty_account_response import CreateLoyaltyAccountResponse
from ...requests.search_loyalty_accounts_request_loyalty_account_query import (
    SearchLoyaltyAccountsRequestLoyaltyAccountQueryParams,
)
from ...types.search_loyalty_accounts_response import SearchLoyaltyAccountsResponse
from ...types.get_loyalty_account_response import GetLoyaltyAccountResponse
from ...requests.loyalty_event_accumulate_points import LoyaltyEventAccumulatePointsParams
from ...types.accumulate_loyalty_points_response import AccumulateLoyaltyPointsResponse
from ...requests.loyalty_event_adjust_points import LoyaltyEventAdjustPointsParams
from ...types.adjust_loyalty_points_response import AdjustLoyaltyPointsResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawAccountsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AccountsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAccountsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAccountsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAccountsClient
        """
        return self._raw_client

    def create(
        self,
        *,
        loyalty_account: LoyaltyAccountParams,
        idempotency_key: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateLoyaltyAccountResponse:
        """
        Creates a loyalty account. To create a loyalty account, you must provide the `program_id` and a `mapping` with the `phone_number` of the buyer.

        Parameters
        ----------
        loyalty_account : LoyaltyAccountParams
            The loyalty account to create.

        idempotency_key : str
            A unique string that identifies this `CreateLoyaltyAccount` request.
            Keys can be any valid string, but must be unique for every request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateLoyaltyAccountResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.loyalty.accounts.create(
            loyalty_account={
                "program_id": "d619f755-2d17-41f3-990d-c04ecedd64dd",
                "mapping": {"phone_number": "+14155551234"},
            },
            idempotency_key="ec78c477-b1c3-4899-a209-a4e71337c996",
        )
        """
        response = self._raw_client.create(
            loyalty_account=loyalty_account, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    def search(
        self,
        *,
        query: typing.Optional[SearchLoyaltyAccountsRequestLoyaltyAccountQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchLoyaltyAccountsResponse:
        """
        Searches for loyalty accounts in a loyalty program.

        You can search for a loyalty account using the phone number or customer ID associated with the account. To return all loyalty accounts, specify an empty `query` object or omit it entirely.

        Search results are sorted by `created_at` in ascending order.

        Parameters
        ----------
        query : typing.Optional[SearchLoyaltyAccountsRequestLoyaltyAccountQueryParams]
            The search criteria for the request.

        limit : typing.Optional[int]
            The maximum number of results to include in the response. The default value is 30.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to
            this endpoint. Provide this to retrieve the next set of
            results for the original query.

            For more information,
            see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchLoyaltyAccountsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.loyalty.accounts.search(
            query={"mappings": [{"phone_number": "+14155551234"}]},
            limit=10,
        )
        """
        response = self._raw_client.search(query=query, limit=limit, cursor=cursor, request_options=request_options)
        return response.data

    def get(
        self, account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetLoyaltyAccountResponse:
        """
        Retrieves a loyalty account.

        Parameters
        ----------
        account_id : str
            The ID of the [loyalty account](entity:LoyaltyAccount) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetLoyaltyAccountResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.loyalty.accounts.get(
            account_id="account_id",
        )
        """
        response = self._raw_client.get(account_id, request_options=request_options)
        return response.data

    def accumulate_points(
        self,
        account_id: str,
        *,
        accumulate_points: LoyaltyEventAccumulatePointsParams,
        idempotency_key: str,
        location_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AccumulateLoyaltyPointsResponse:
        """
        Adds points earned from a purchase to a [loyalty account](entity:LoyaltyAccount).

        - If you are using the Orders API to manage orders, provide the `order_id`. Square reads the order
        to compute the points earned from both the base loyalty program and an associated
        [loyalty promotion](entity:LoyaltyPromotion). For purchases that qualify for multiple accrual
        rules, Square computes points based on the accrual rule that grants the most points.
        For purchases that qualify for multiple promotions, Square computes points based on the most
        recently created promotion. A purchase must first qualify for program points to be eligible for promotion points.

        - If you are not using the Orders API to manage orders, provide `points` with the number of points to add.
        You must first perform a client-side computation of the points earned from the loyalty program and
        loyalty promotion. For spend-based and visit-based programs, you can call [CalculateLoyaltyPoints](api-endpoint:Loyalty-CalculateLoyaltyPoints)
        to compute the points earned from the base loyalty program. For information about computing points earned from a loyalty promotion, see
        [Calculating promotion points](https://developer.squareup.com/docs/loyalty-api/loyalty-promotions#calculate-promotion-points).

        Parameters
        ----------
        account_id : str
            The ID of the target [loyalty account](entity:LoyaltyAccount).

        accumulate_points : LoyaltyEventAccumulatePointsParams
            The points to add to the account.
            If you are using the Orders API to manage orders, specify the order ID.
            Otherwise, specify the points to add.

        idempotency_key : str
            A unique string that identifies the `AccumulateLoyaltyPoints` request.
            Keys can be any valid string but must be unique for every request.

        location_id : str
            The [location](entity:Location) where the purchase was made.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccumulateLoyaltyPointsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.loyalty.accounts.accumulate_points(
            account_id="account_id",
            accumulate_points={"order_id": "RFZfrdtm3mhO1oGzf5Cx7fEMsmGZY"},
            idempotency_key="58b90739-c3e8-4b11-85f7-e636d48d72cb",
            location_id="P034NEENMD09F",
        )
        """
        response = self._raw_client.accumulate_points(
            account_id,
            accumulate_points=accumulate_points,
            idempotency_key=idempotency_key,
            location_id=location_id,
            request_options=request_options,
        )
        return response.data

    def adjust(
        self,
        account_id: str,
        *,
        idempotency_key: str,
        adjust_points: LoyaltyEventAdjustPointsParams,
        allow_negative_balance: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AdjustLoyaltyPointsResponse:
        """
        Adds points to or subtracts points from a buyer's account.

        Use this endpoint only when you need to manually adjust points. Otherwise, in your application flow, you call
        [AccumulateLoyaltyPoints](api-endpoint:Loyalty-AccumulateLoyaltyPoints)
        to add points when a buyer pays for the purchase.

        Parameters
        ----------
        account_id : str
            The ID of the target [loyalty account](entity:LoyaltyAccount).

        idempotency_key : str
            A unique string that identifies this `AdjustLoyaltyPoints` request.
            Keys can be any valid string, but must be unique for every request.

        adjust_points : LoyaltyEventAdjustPointsParams
            The points to add or subtract and the reason for the adjustment. To add points, specify a positive integer.
            To subtract points, specify a negative integer.

        allow_negative_balance : typing.Optional[bool]
            Indicates whether to allow a negative adjustment to result in a negative balance. If `true`, a negative
            balance is allowed when subtracting points. If `false`, Square returns a `BAD_REQUEST` error when subtracting
            the specified number of points would result in a negative balance. The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AdjustLoyaltyPointsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.loyalty.accounts.adjust(
            account_id="account_id",
            idempotency_key="bc29a517-3dc9-450e-aa76-fae39ee849d1",
            adjust_points={"points": 10, "reason": "Complimentary points"},
        )
        """
        response = self._raw_client.adjust(
            account_id,
            idempotency_key=idempotency_key,
            adjust_points=adjust_points,
            allow_negative_balance=allow_negative_balance,
            request_options=request_options,
        )
        return response.data


class AsyncAccountsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAccountsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAccountsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAccountsClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        loyalty_account: LoyaltyAccountParams,
        idempotency_key: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateLoyaltyAccountResponse:
        """
        Creates a loyalty account. To create a loyalty account, you must provide the `program_id` and a `mapping` with the `phone_number` of the buyer.

        Parameters
        ----------
        loyalty_account : LoyaltyAccountParams
            The loyalty account to create.

        idempotency_key : str
            A unique string that identifies this `CreateLoyaltyAccount` request.
            Keys can be any valid string, but must be unique for every request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateLoyaltyAccountResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.loyalty.accounts.create(
                loyalty_account={
                    "program_id": "d619f755-2d17-41f3-990d-c04ecedd64dd",
                    "mapping": {"phone_number": "+14155551234"},
                },
                idempotency_key="ec78c477-b1c3-4899-a209-a4e71337c996",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            loyalty_account=loyalty_account, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    async def search(
        self,
        *,
        query: typing.Optional[SearchLoyaltyAccountsRequestLoyaltyAccountQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchLoyaltyAccountsResponse:
        """
        Searches for loyalty accounts in a loyalty program.

        You can search for a loyalty account using the phone number or customer ID associated with the account. To return all loyalty accounts, specify an empty `query` object or omit it entirely.

        Search results are sorted by `created_at` in ascending order.

        Parameters
        ----------
        query : typing.Optional[SearchLoyaltyAccountsRequestLoyaltyAccountQueryParams]
            The search criteria for the request.

        limit : typing.Optional[int]
            The maximum number of results to include in the response. The default value is 30.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to
            this endpoint. Provide this to retrieve the next set of
            results for the original query.

            For more information,
            see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchLoyaltyAccountsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.loyalty.accounts.search(
                query={"mappings": [{"phone_number": "+14155551234"}]},
                limit=10,
            )


        asyncio.run(main())
        """
        response = await self._raw_client.search(
            query=query, limit=limit, cursor=cursor, request_options=request_options
        )
        return response.data

    async def get(
        self, account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetLoyaltyAccountResponse:
        """
        Retrieves a loyalty account.

        Parameters
        ----------
        account_id : str
            The ID of the [loyalty account](entity:LoyaltyAccount) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetLoyaltyAccountResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.loyalty.accounts.get(
                account_id="account_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(account_id, request_options=request_options)
        return response.data

    async def accumulate_points(
        self,
        account_id: str,
        *,
        accumulate_points: LoyaltyEventAccumulatePointsParams,
        idempotency_key: str,
        location_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AccumulateLoyaltyPointsResponse:
        """
        Adds points earned from a purchase to a [loyalty account](entity:LoyaltyAccount).

        - If you are using the Orders API to manage orders, provide the `order_id`. Square reads the order
        to compute the points earned from both the base loyalty program and an associated
        [loyalty promotion](entity:LoyaltyPromotion). For purchases that qualify for multiple accrual
        rules, Square computes points based on the accrual rule that grants the most points.
        For purchases that qualify for multiple promotions, Square computes points based on the most
        recently created promotion. A purchase must first qualify for program points to be eligible for promotion points.

        - If you are not using the Orders API to manage orders, provide `points` with the number of points to add.
        You must first perform a client-side computation of the points earned from the loyalty program and
        loyalty promotion. For spend-based and visit-based programs, you can call [CalculateLoyaltyPoints](api-endpoint:Loyalty-CalculateLoyaltyPoints)
        to compute the points earned from the base loyalty program. For information about computing points earned from a loyalty promotion, see
        [Calculating promotion points](https://developer.squareup.com/docs/loyalty-api/loyalty-promotions#calculate-promotion-points).

        Parameters
        ----------
        account_id : str
            The ID of the target [loyalty account](entity:LoyaltyAccount).

        accumulate_points : LoyaltyEventAccumulatePointsParams
            The points to add to the account.
            If you are using the Orders API to manage orders, specify the order ID.
            Otherwise, specify the points to add.

        idempotency_key : str
            A unique string that identifies the `AccumulateLoyaltyPoints` request.
            Keys can be any valid string but must be unique for every request.

        location_id : str
            The [location](entity:Location) where the purchase was made.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccumulateLoyaltyPointsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.loyalty.accounts.accumulate_points(
                account_id="account_id",
                accumulate_points={"order_id": "RFZfrdtm3mhO1oGzf5Cx7fEMsmGZY"},
                idempotency_key="58b90739-c3e8-4b11-85f7-e636d48d72cb",
                location_id="P034NEENMD09F",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.accumulate_points(
            account_id,
            accumulate_points=accumulate_points,
            idempotency_key=idempotency_key,
            location_id=location_id,
            request_options=request_options,
        )
        return response.data

    async def adjust(
        self,
        account_id: str,
        *,
        idempotency_key: str,
        adjust_points: LoyaltyEventAdjustPointsParams,
        allow_negative_balance: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AdjustLoyaltyPointsResponse:
        """
        Adds points to or subtracts points from a buyer's account.

        Use this endpoint only when you need to manually adjust points. Otherwise, in your application flow, you call
        [AccumulateLoyaltyPoints](api-endpoint:Loyalty-AccumulateLoyaltyPoints)
        to add points when a buyer pays for the purchase.

        Parameters
        ----------
        account_id : str
            The ID of the target [loyalty account](entity:LoyaltyAccount).

        idempotency_key : str
            A unique string that identifies this `AdjustLoyaltyPoints` request.
            Keys can be any valid string, but must be unique for every request.

        adjust_points : LoyaltyEventAdjustPointsParams
            The points to add or subtract and the reason for the adjustment. To add points, specify a positive integer.
            To subtract points, specify a negative integer.

        allow_negative_balance : typing.Optional[bool]
            Indicates whether to allow a negative adjustment to result in a negative balance. If `true`, a negative
            balance is allowed when subtracting points. If `false`, Square returns a `BAD_REQUEST` error when subtracting
            the specified number of points would result in a negative balance. The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AdjustLoyaltyPointsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.loyalty.accounts.adjust(
                account_id="account_id",
                idempotency_key="bc29a517-3dc9-450e-aa76-fae39ee849d1",
                adjust_points={"points": 10, "reason": "Complimentary points"},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.adjust(
            account_id,
            idempotency_key=idempotency_key,
            adjust_points=adjust_points,
            allow_negative_balance=allow_negative_balance,
            request_options=request_options,
        )
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .address import AddressParams
from .customer_tax_ids import CustomerTaxIdsParams


class BulkUpdateCustomerDataParams(typing_extensions.TypedDict):
    """
    Defines the customer data provided in individual update requests for a
    [BulkUpdateCustomers](api-endpoint:Customers-BulkUpdateCustomers) operation.
    """

    given_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The given name (that is, the first name) associated with the customer profile.
    """

    family_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The family name (that is, the last name) associated with the customer profile.
    """

    company_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A business name associated with the customer profile.
    """

    nickname: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A nickname for the customer profile.
    """

    email_address: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The email address associated with the customer profile.
    """

    address: typing_extensions.NotRequired[AddressParams]
    """
    The physical address associated with the customer profile. For maximum length constraints,
    see [Customer addresses](https://developer.squareup.com/docs/customers-api/use-the-api/keep-records#address).
    The `first_name` and `last_name` fields are ignored if they are present in the request.
    """

    phone_number: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The phone number associated with the customer profile. The phone number must be valid
    and can contain 9–16 digits, with an optional `+` prefix and country code. For more information,
    see [Customer phone numbers](https://developer.squareup.com/docs/customers-api/use-the-api/keep-records#phone-number).
    """

    reference_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    An optional second ID used to associate the customer profile with an
    entity in another system.
    """

    note: typing_extensions.NotRequired[typing.Optional[str]]
    """
    An custom note associates with the customer profile.
    """

    birthday: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The birthday associated with the customer profile, in `YYYY-MM-DD` or `MM-DD` format.
    For example, specify `1998-09-21` for September 21, 1998, or `09-21` for September 21.
    Birthdays are returned in `YYYY-MM-DD` format, where `YYYY` is the specified birth year or
    `0000` if a birth year is not specified.
    """

    tax_ids: typing_extensions.NotRequired[CustomerTaxIdsParams]
    """
    The tax ID associated with the customer profile. This field is available only for
    customers of sellers in EU countries or the United Kingdom. For more information, see
    [Customer tax IDs](https://developer.squareup.com/docs/customers-api/what-it-does#customer-tax-ids).
    """

    version: typing_extensions.NotRequired[int]
    """
    The current version of the customer profile.
    
    As a best practice, you should include this field to enable
    [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
    control.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class PaymentBalanceActivityReserveReleaseDetailParams(typing_extensions.TypedDict):
    payment_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the payment associated with this activity.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.subscription_pricing_type import SubscriptionPricingType
import typing
from .money import MoneyParams


class SubscriptionPricingParams(typing_extensions.TypedDict):
    """
    Describes the pricing for the subscription.
    """

    type: typing_extensions.NotRequired[SubscriptionPricingType]
    """
    RELATIVE or STATIC
    See [SubscriptionPricingType](#type-subscriptionpricingtype) for possible values
    """

    discount_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The ids of the discount catalog objects
    """

    price_money: typing_extensions.NotRequired[MoneyParams]
    """
    The price of the subscription, if STATIC
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.device_component_details_external_power import DeviceComponentDetailsExternalPower


class DeviceComponentDetailsBatteryDetailsParams(typing_extensions.TypedDict):
    visible_percent: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The battery charge percentage as displayed on the device.
    """

    external_power: typing_extensions.NotRequired[DeviceComponentDetailsExternalPower]
    """
    The status of external_power.
    See [ExternalPower](#type-externalpower) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .team_member_status import TeamMemberStatus
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchTeamMembersFilter(UncheckedBaseModel):
    """
    Represents a filter used in a search for `TeamMember` objects. `AND` logic is applied
    between the individual fields, and `OR` logic is applied within list-based fields.
    For example, setting this filter value:
    ```
    filter = (locations_ids = ["A", "B"], status = ACTIVE)
    ```
    returns only active team members assigned to either location "A" or "B".
    """

    location_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    When present, filters by team members assigned to the specified locations.
    When empty, includes team members assigned to any location.
    """

    status: typing.Optional[TeamMemberStatus] = pydantic.Field(default=None)
    """
    When present, filters by team members who match the given status.
    When empty, includes team members of all statuses.
    See [TeamMemberStatus](#type-teammemberstatus) for possible values
    """

    is_owner: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When present and set to true, returns the team member who is the owner of the Square account.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions


class BulkDeleteLocationCustomAttributesRequestLocationCustomAttributeDeleteRequestParams(typing_extensions.TypedDict):
    """
    Represents an individual delete request in a [BulkDeleteLocationCustomAttributes](api-endpoint:LocationCustomAttributes-BulkDeleteLocationCustomAttributes)
    request. An individual request contains an optional ID of the associated custom attribute definition
    and optional key of the associated custom attribute definition.
    """

    key: typing_extensions.NotRequired[str]
    """
    The key of the associated custom attribute definition.
    Represented as a qualified key if the requesting app is not the definition owner.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawDisputesClient
from .evidence.client import EvidenceClient
from ..types.dispute_state import DisputeState
from ..core.request_options import RequestOptions
from ..core.pagination import SyncPager
from ..types.dispute import Dispute
from ..types.list_disputes_response import ListDisputesResponse
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.get_dispute_response import GetDisputeResponse
from ..types.accept_dispute_response import AcceptDisputeResponse
from ..requests.create_dispute_evidence_file_request import CreateDisputeEvidenceFileRequestParams
from .. import core
from ..types.create_dispute_evidence_file_response import CreateDisputeEvidenceFileResponse
from ..types.dispute_evidence_type import DisputeEvidenceType
from ..types.create_dispute_evidence_text_response import CreateDisputeEvidenceTextResponse
from ..types.submit_evidence_response import SubmitEvidenceResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawDisputesClient
from .evidence.client import AsyncEvidenceClient
from ..core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DisputesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDisputesClient(client_wrapper=client_wrapper)
        self.evidence = EvidenceClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDisputesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDisputesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        states: typing.Optional[DisputeState] = None,
        location_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Dispute]:
        """
        Returns a list of disputes associated with a particular account.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        states : typing.Optional[DisputeState]
            The dispute states used to filter the result. If not specified, the endpoint returns all disputes.

        location_id : typing.Optional[str]
            The ID of the location for which to return a list of disputes.
            If not specified, the endpoint returns disputes associated with all locations.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Dispute]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.disputes.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/disputes",
            method="GET",
            params={
                "cursor": cursor,
                "states": states,
                "location_id": location_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListDisputesResponse,
                    construct_type(
                        type_=ListDisputesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    states=states,
                    location_id=location_id,
                    request_options=request_options,
                )
                _items = _parsed_response.disputes
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, dispute_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetDisputeResponse:
        """
        Returns details about a specific dispute.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute you want more details about.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDisputeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.disputes.get(
            dispute_id="dispute_id",
        )
        """
        response = self._raw_client.get(dispute_id, request_options=request_options)
        return response.data

    def accept(
        self, dispute_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AcceptDisputeResponse:
        """
        Accepts the loss on a dispute. Square returns the disputed amount to the cardholder and
        updates the dispute state to ACCEPTED.

        Square debits the disputed amount from the seller’s Square account. If the Square account
        does not have sufficient funds, Square debits the associated bank account.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute you want to accept.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AcceptDisputeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.disputes.accept(
            dispute_id="dispute_id",
        )
        """
        response = self._raw_client.accept(dispute_id, request_options=request_options)
        return response.data

    def create_evidence_file(
        self,
        dispute_id: str,
        *,
        request: typing.Optional[CreateDisputeEvidenceFileRequestParams] = OMIT,
        image_file: typing.Optional[core.File] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateDisputeEvidenceFileResponse:
        """
        Uploads a file to use as evidence in a dispute challenge. The endpoint accepts HTTP
        multipart/form-data file uploads in HEIC, HEIF, JPEG, PDF, PNG, and TIFF formats.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute for which you want to upload evidence.

        request : typing.Optional[CreateDisputeEvidenceFileRequestParams]

        image_file : typing.Optional[core.File]
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateDisputeEvidenceFileResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.disputes.create_evidence_file(
            dispute_id="dispute_id",
        )
        """
        response = self._raw_client.create_evidence_file(
            dispute_id, request=request, image_file=image_file, request_options=request_options
        )
        return response.data

    def create_evidence_text(
        self,
        dispute_id: str,
        *,
        idempotency_key: str,
        evidence_text: str,
        evidence_type: typing.Optional[DisputeEvidenceType] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateDisputeEvidenceTextResponse:
        """
        Uploads text to use as evidence for a dispute challenge.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute for which you want to upload evidence.

        idempotency_key : str
            A unique key identifying the request. For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        evidence_text : str
            The evidence string.

        evidence_type : typing.Optional[DisputeEvidenceType]
            The type of evidence you are uploading.
            See [DisputeEvidenceType](#type-disputeevidencetype) for possible values

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateDisputeEvidenceTextResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.disputes.create_evidence_text(
            dispute_id="dispute_id",
            idempotency_key="ed3ee3933d946f1514d505d173c82648",
            evidence_type="TRACKING_NUMBER",
            evidence_text="1Z8888888888888888",
        )
        """
        response = self._raw_client.create_evidence_text(
            dispute_id,
            idempotency_key=idempotency_key,
            evidence_text=evidence_text,
            evidence_type=evidence_type,
            request_options=request_options,
        )
        return response.data

    def submit_evidence(
        self, dispute_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SubmitEvidenceResponse:
        """
        Submits evidence to the cardholder's bank.

        The evidence submitted by this endpoint includes evidence uploaded
        using the [CreateDisputeEvidenceFile](api-endpoint:Disputes-CreateDisputeEvidenceFile) and
        [CreateDisputeEvidenceText](api-endpoint:Disputes-CreateDisputeEvidenceText) endpoints and
        evidence automatically provided by Square, when available. Evidence cannot be removed from
        a dispute after submission.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute for which you want to submit evidence.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubmitEvidenceResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.disputes.submit_evidence(
            dispute_id="dispute_id",
        )
        """
        response = self._raw_client.submit_evidence(dispute_id, request_options=request_options)
        return response.data


class AsyncDisputesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDisputesClient(client_wrapper=client_wrapper)
        self.evidence = AsyncEvidenceClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDisputesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDisputesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        states: typing.Optional[DisputeState] = None,
        location_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Dispute]:
        """
        Returns a list of disputes associated with a particular account.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        states : typing.Optional[DisputeState]
            The dispute states used to filter the result. If not specified, the endpoint returns all disputes.

        location_id : typing.Optional[str]
            The ID of the location for which to return a list of disputes.
            If not specified, the endpoint returns disputes associated with all locations.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Dispute]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.disputes.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/disputes",
            method="GET",
            params={
                "cursor": cursor,
                "states": states,
                "location_id": location_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListDisputesResponse,
                    construct_type(
                        type_=ListDisputesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    states=states,
                    location_id=location_id,
                    request_options=request_options,
                )
                _items = _parsed_response.disputes
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, dispute_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetDisputeResponse:
        """
        Returns details about a specific dispute.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute you want more details about.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDisputeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.disputes.get(
                dispute_id="dispute_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(dispute_id, request_options=request_options)
        return response.data

    async def accept(
        self, dispute_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AcceptDisputeResponse:
        """
        Accepts the loss on a dispute. Square returns the disputed amount to the cardholder and
        updates the dispute state to ACCEPTED.

        Square debits the disputed amount from the seller’s Square account. If the Square account
        does not have sufficient funds, Square debits the associated bank account.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute you want to accept.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AcceptDisputeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.disputes.accept(
                dispute_id="dispute_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.accept(dispute_id, request_options=request_options)
        return response.data

    async def create_evidence_file(
        self,
        dispute_id: str,
        *,
        request: typing.Optional[CreateDisputeEvidenceFileRequestParams] = OMIT,
        image_file: typing.Optional[core.File] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateDisputeEvidenceFileResponse:
        """
        Uploads a file to use as evidence in a dispute challenge. The endpoint accepts HTTP
        multipart/form-data file uploads in HEIC, HEIF, JPEG, PDF, PNG, and TIFF formats.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute for which you want to upload evidence.

        request : typing.Optional[CreateDisputeEvidenceFileRequestParams]

        image_file : typing.Optional[core.File]
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateDisputeEvidenceFileResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.disputes.create_evidence_file(
                dispute_id="dispute_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create_evidence_file(
            dispute_id, request=request, image_file=image_file, request_options=request_options
        )
        return response.data

    async def create_evidence_text(
        self,
        dispute_id: str,
        *,
        idempotency_key: str,
        evidence_text: str,
        evidence_type: typing.Optional[DisputeEvidenceType] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateDisputeEvidenceTextResponse:
        """
        Uploads text to use as evidence for a dispute challenge.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute for which you want to upload evidence.

        idempotency_key : str
            A unique key identifying the request. For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        evidence_text : str
            The evidence string.

        evidence_type : typing.Optional[DisputeEvidenceType]
            The type of evidence you are uploading.
            See [DisputeEvidenceType](#type-disputeevidencetype) for possible values

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateDisputeEvidenceTextResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.disputes.create_evidence_text(
                dispute_id="dispute_id",
                idempotency_key="ed3ee3933d946f1514d505d173c82648",
                evidence_type="TRACKING_NUMBER",
                evidence_text="1Z8888888888888888",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create_evidence_text(
            dispute_id,
            idempotency_key=idempotency_key,
            evidence_text=evidence_text,
            evidence_type=evidence_type,
            request_options=request_options,
        )
        return response.data

    async def submit_evidence(
        self, dispute_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SubmitEvidenceResponse:
        """
        Submits evidence to the cardholder's bank.

        The evidence submitted by this endpoint includes evidence uploaded
        using the [CreateDisputeEvidenceFile](api-endpoint:Disputes-CreateDisputeEvidenceFile) and
        [CreateDisputeEvidenceText](api-endpoint:Disputes-CreateDisputeEvidenceText) endpoints and
        evidence automatically provided by Square, when available. Evidence cannot be removed from
        a dispute after submission.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute for which you want to submit evidence.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubmitEvidenceResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.disputes.submit_evidence(
                dispute_id="dispute_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.submit_evidence(dispute_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawCustomAttributesClient
from ...requests.bulk_delete_order_custom_attributes_request_delete_custom_attribute import (
    BulkDeleteOrderCustomAttributesRequestDeleteCustomAttributeParams,
)
from ...core.request_options import RequestOptions
from ...types.bulk_delete_order_custom_attributes_response import BulkDeleteOrderCustomAttributesResponse
from ...requests.bulk_upsert_order_custom_attributes_request_upsert_custom_attribute import (
    BulkUpsertOrderCustomAttributesRequestUpsertCustomAttributeParams,
)
from ...types.bulk_upsert_order_custom_attributes_response import BulkUpsertOrderCustomAttributesResponse
from ...types.visibility_filter import VisibilityFilter
from ...core.pagination import SyncPager
from ...types.custom_attribute import CustomAttribute
from ...core.jsonable_encoder import jsonable_encoder
from ...types.list_order_custom_attributes_response import ListOrderCustomAttributesResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.retrieve_order_custom_attribute_response import RetrieveOrderCustomAttributeResponse
from ...requests.custom_attribute import CustomAttributeParams
from ...types.upsert_order_custom_attribute_response import UpsertOrderCustomAttributeResponse
from ...types.delete_order_custom_attribute_response import DeleteOrderCustomAttributeResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawCustomAttributesClient
from ...core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomAttributesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCustomAttributesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCustomAttributesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCustomAttributesClient
        """
        return self._raw_client

    def batch_delete(
        self,
        *,
        values: typing.Dict[str, BulkDeleteOrderCustomAttributesRequestDeleteCustomAttributeParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkDeleteOrderCustomAttributesResponse:
        """
        Deletes order [custom attributes](entity:CustomAttribute) as a bulk operation.

        Use this endpoint to delete one or more custom attributes from one or more orders.
        A custom attribute is based on a custom attribute definition in a Square seller account.  (To create a
        custom attribute definition, use the [CreateOrderCustomAttributeDefinition](api-endpoint:OrderCustomAttributes-CreateOrderCustomAttributeDefinition) endpoint.)

        This `BulkDeleteOrderCustomAttributes` endpoint accepts a map of 1 to 25 individual delete
        requests and returns a map of individual delete responses. Each delete request has a unique ID
        and provides an order ID and custom attribute. Each delete response is returned with the ID
        of the corresponding request.

        To delete a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BulkDeleteOrderCustomAttributesRequestDeleteCustomAttributeParams]
            A map of requests that correspond to individual delete operations for custom attributes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkDeleteOrderCustomAttributesResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.orders.custom_attributes.batch_delete(
            values={
                "cover-count": {
                    "key": "cover-count",
                    "order_id": "7BbXGEIWNldxAzrtGf9GPVZTwZ4F",
                },
                "table-number": {
                    "key": "table-number",
                    "order_id": "7BbXGEIWNldxAzrtGf9GPVZTwZ4F",
                },
            },
        )
        """
        response = self._raw_client.batch_delete(values=values, request_options=request_options)
        return response.data

    def batch_upsert(
        self,
        *,
        values: typing.Dict[str, BulkUpsertOrderCustomAttributesRequestUpsertCustomAttributeParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkUpsertOrderCustomAttributesResponse:
        """
        Creates or updates order [custom attributes](entity:CustomAttribute) as a bulk operation.

        Use this endpoint to delete one or more custom attributes from one or more orders.
        A custom attribute is based on a custom attribute definition in a Square seller account.  (To create a
        custom attribute definition, use the [CreateOrderCustomAttributeDefinition](api-endpoint:OrderCustomAttributes-CreateOrderCustomAttributeDefinition) endpoint.)

        This `BulkUpsertOrderCustomAttributes` endpoint accepts a map of 1 to 25 individual upsert
        requests and returns a map of individual upsert responses. Each upsert request has a unique ID
        and provides an order ID and custom attribute. Each upsert response is returned with the ID
        of the corresponding request.

        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BulkUpsertOrderCustomAttributesRequestUpsertCustomAttributeParams]
            A map of requests that correspond to individual upsert operations for custom attributes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkUpsertOrderCustomAttributesResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.orders.custom_attributes.batch_upsert(
            values={
                "cover-count": {
                    "custom_attribute": {
                        "key": "cover-count",
                        "value": "6",
                        "version": 2,
                    },
                    "order_id": "7BbXGEIWNldxAzrtGf9GPVZTwZ4F",
                },
                "table-number": {
                    "custom_attribute": {
                        "key": "table-number",
                        "value": "11",
                        "version": 4,
                    },
                    "order_id": "7BbXGEIWNldxAzrtGf9GPVZTwZ4F",
                },
            },
        )
        """
        response = self._raw_client.batch_upsert(values=values, request_options=request_options)
        return response.data

    def list(
        self,
        order_id: str,
        *,
        visibility_filter: typing.Optional[VisibilityFilter] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        with_definitions: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[CustomAttribute]:
        """
        Lists the [custom attributes](entity:CustomAttribute) associated with an order.

        You can use the `with_definitions` query parameter to also retrieve custom attribute definitions
        in the same call.

        When all response pages are retrieved, the results include all custom attributes that are
        visible to the requesting application, including those that are owned by other applications
        and set to `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        order_id : str
            The ID of the target [order](entity:Order).

        visibility_filter : typing.Optional[VisibilityFilter]
            Requests that all of the custom attributes be returned, or only those that are read-only or read-write.

        cursor : typing.Optional[str]
            The cursor returned in the paged response from the previous call to this endpoint.
            Provide this cursor to retrieve the next page of results for your original request.
            For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        limit : typing.Optional[int]
            The maximum number of results to return in a single paged response. This limit is advisory.
            The response might contain more or fewer results. The minimum value is 1 and the maximum value is 100.
            The default value is 20.
            For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        with_definitions : typing.Optional[bool]
            Indicates whether to return the [custom attribute definition](entity:CustomAttributeDefinition) in the `definition` field of each
            custom attribute. Set this parameter to `true` to get the name and description of each custom attribute,
            information about the data type, or other definition details. The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[CustomAttribute]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.orders.custom_attributes.list(
            order_id="order_id",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            f"v2/orders/{jsonable_encoder(order_id)}/custom-attributes",
            method="GET",
            params={
                "visibility_filter": visibility_filter,
                "cursor": cursor,
                "limit": limit,
                "with_definitions": with_definitions,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListOrderCustomAttributesResponse,
                    construct_type(
                        type_=ListOrderCustomAttributesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    order_id,
                    visibility_filter=visibility_filter,
                    cursor=_parsed_next,
                    limit=limit,
                    with_definitions=with_definitions,
                    request_options=request_options,
                )
                _items = _parsed_response.custom_attributes
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self,
        order_id: str,
        custom_attribute_key: str,
        *,
        version: typing.Optional[int] = None,
        with_definition: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RetrieveOrderCustomAttributeResponse:
        """
        Retrieves a [custom attribute](entity:CustomAttribute) associated with an order.

        You can use the `with_definition` query parameter to also retrieve the custom attribute definition
        in the same call.

        To retrieve a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        order_id : str
            The ID of the target [order](entity:Order).

        custom_attribute_key : str
            The key of the custom attribute to retrieve.  This key must match the key of an
            existing custom attribute definition.

        version : typing.Optional[int]
            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include this optional field and specify the current version of the custom attribute.

        with_definition : typing.Optional[bool]
            Indicates whether to return the [custom attribute definition](entity:CustomAttributeDefinition) in the `definition` field of each
            custom attribute. Set this parameter to `true` to get the name and description of each custom attribute,
            information about the data type, or other definition details. The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveOrderCustomAttributeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.orders.custom_attributes.get(
            order_id="order_id",
            custom_attribute_key="custom_attribute_key",
        )
        """
        response = self._raw_client.get(
            order_id,
            custom_attribute_key,
            version=version,
            with_definition=with_definition,
            request_options=request_options,
        )
        return response.data

    def upsert(
        self,
        order_id: str,
        custom_attribute_key: str,
        *,
        custom_attribute: CustomAttributeParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpsertOrderCustomAttributeResponse:
        """
        Creates or updates a [custom attribute](entity:CustomAttribute) for an order.

        Use this endpoint to set the value of a custom attribute for a specific order.
        A custom attribute is based on a custom attribute definition in a Square seller account. (To create a
        custom attribute definition, use the [CreateOrderCustomAttributeDefinition](api-endpoint:OrderCustomAttributes-CreateOrderCustomAttributeDefinition) endpoint.)

        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        order_id : str
            The ID of the target [order](entity:Order).

        custom_attribute_key : str
            The key of the custom attribute to create or update.  This key must match the key
            of an existing custom attribute definition.

        custom_attribute : CustomAttributeParams
            The custom attribute to create or update, with the following fields:

            - `value`. This value must conform to the `schema` specified by the definition.
            For more information, see [Value data types](https://developer.squareup.com/docs/customer-custom-attributes-api/custom-attributes#value-data-types).

            - `version`. To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include this optional field and specify the current version of the custom attribute.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency.
            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpsertOrderCustomAttributeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.orders.custom_attributes.upsert(
            order_id="order_id",
            custom_attribute_key="custom_attribute_key",
            custom_attribute={"key": "table-number", "value": "42", "version": 1},
        )
        """
        response = self._raw_client.upsert(
            order_id,
            custom_attribute_key,
            custom_attribute=custom_attribute,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    def delete(
        self, order_id: str, custom_attribute_key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteOrderCustomAttributeResponse:
        """
        Deletes a [custom attribute](entity:CustomAttribute) associated with a customer profile.

        To delete a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        order_id : str
            The ID of the target [order](entity:Order).

        custom_attribute_key : str
            The key of the custom attribute to delete.  This key must match the key of an
            existing custom attribute definition.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteOrderCustomAttributeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.orders.custom_attributes.delete(
            order_id="order_id",
            custom_attribute_key="custom_attribute_key",
        )
        """
        response = self._raw_client.delete(order_id, custom_attribute_key, request_options=request_options)
        return response.data


class AsyncCustomAttributesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCustomAttributesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCustomAttributesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCustomAttributesClient
        """
        return self._raw_client

    async def batch_delete(
        self,
        *,
        values: typing.Dict[str, BulkDeleteOrderCustomAttributesRequestDeleteCustomAttributeParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkDeleteOrderCustomAttributesResponse:
        """
        Deletes order [custom attributes](entity:CustomAttribute) as a bulk operation.

        Use this endpoint to delete one or more custom attributes from one or more orders.
        A custom attribute is based on a custom attribute definition in a Square seller account.  (To create a
        custom attribute definition, use the [CreateOrderCustomAttributeDefinition](api-endpoint:OrderCustomAttributes-CreateOrderCustomAttributeDefinition) endpoint.)

        This `BulkDeleteOrderCustomAttributes` endpoint accepts a map of 1 to 25 individual delete
        requests and returns a map of individual delete responses. Each delete request has a unique ID
        and provides an order ID and custom attribute. Each delete response is returned with the ID
        of the corresponding request.

        To delete a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BulkDeleteOrderCustomAttributesRequestDeleteCustomAttributeParams]
            A map of requests that correspond to individual delete operations for custom attributes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkDeleteOrderCustomAttributesResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.orders.custom_attributes.batch_delete(
                values={
                    "cover-count": {
                        "key": "cover-count",
                        "order_id": "7BbXGEIWNldxAzrtGf9GPVZTwZ4F",
                    },
                    "table-number": {
                        "key": "table-number",
                        "order_id": "7BbXGEIWNldxAzrtGf9GPVZTwZ4F",
                    },
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.batch_delete(values=values, request_options=request_options)
        return response.data

    async def batch_upsert(
        self,
        *,
        values: typing.Dict[str, BulkUpsertOrderCustomAttributesRequestUpsertCustomAttributeParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkUpsertOrderCustomAttributesResponse:
        """
        Creates or updates order [custom attributes](entity:CustomAttribute) as a bulk operation.

        Use this endpoint to delete one or more custom attributes from one or more orders.
        A custom attribute is based on a custom attribute definition in a Square seller account.  (To create a
        custom attribute definition, use the [CreateOrderCustomAttributeDefinition](api-endpoint:OrderCustomAttributes-CreateOrderCustomAttributeDefinition) endpoint.)

        This `BulkUpsertOrderCustomAttributes` endpoint accepts a map of 1 to 25 individual upsert
        requests and returns a map of individual upsert responses. Each upsert request has a unique ID
        and provides an order ID and custom attribute. Each upsert response is returned with the ID
        of the corresponding request.

        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BulkUpsertOrderCustomAttributesRequestUpsertCustomAttributeParams]
            A map of requests that correspond to individual upsert operations for custom attributes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkUpsertOrderCustomAttributesResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.orders.custom_attributes.batch_upsert(
                values={
                    "cover-count": {
                        "custom_attribute": {
                            "key": "cover-count",
                            "value": "6",
                            "version": 2,
                        },
                        "order_id": "7BbXGEIWNldxAzrtGf9GPVZTwZ4F",
                    },
                    "table-number": {
                        "custom_attribute": {
                            "key": "table-number",
                            "value": "11",
                            "version": 4,
                        },
                        "order_id": "7BbXGEIWNldxAzrtGf9GPVZTwZ4F",
                    },
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.batch_upsert(values=values, request_options=request_options)
        return response.data

    async def list(
        self,
        order_id: str,
        *,
        visibility_filter: typing.Optional[VisibilityFilter] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        with_definitions: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[CustomAttribute]:
        """
        Lists the [custom attributes](entity:CustomAttribute) associated with an order.

        You can use the `with_definitions` query parameter to also retrieve custom attribute definitions
        in the same call.

        When all response pages are retrieved, the results include all custom attributes that are
        visible to the requesting application, including those that are owned by other applications
        and set to `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        order_id : str
            The ID of the target [order](entity:Order).

        visibility_filter : typing.Optional[VisibilityFilter]
            Requests that all of the custom attributes be returned, or only those that are read-only or read-write.

        cursor : typing.Optional[str]
            The cursor returned in the paged response from the previous call to this endpoint.
            Provide this cursor to retrieve the next page of results for your original request.
            For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        limit : typing.Optional[int]
            The maximum number of results to return in a single paged response. This limit is advisory.
            The response might contain more or fewer results. The minimum value is 1 and the maximum value is 100.
            The default value is 20.
            For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        with_definitions : typing.Optional[bool]
            Indicates whether to return the [custom attribute definition](entity:CustomAttributeDefinition) in the `definition` field of each
            custom attribute. Set this parameter to `true` to get the name and description of each custom attribute,
            information about the data type, or other definition details. The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[CustomAttribute]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.orders.custom_attributes.list(
                order_id="order_id",
            )
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            f"v2/orders/{jsonable_encoder(order_id)}/custom-attributes",
            method="GET",
            params={
                "visibility_filter": visibility_filter,
                "cursor": cursor,
                "limit": limit,
                "with_definitions": with_definitions,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListOrderCustomAttributesResponse,
                    construct_type(
                        type_=ListOrderCustomAttributesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    order_id,
                    visibility_filter=visibility_filter,
                    cursor=_parsed_next,
                    limit=limit,
                    with_definitions=with_definitions,
                    request_options=request_options,
                )
                _items = _parsed_response.custom_attributes
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self,
        order_id: str,
        custom_attribute_key: str,
        *,
        version: typing.Optional[int] = None,
        with_definition: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RetrieveOrderCustomAttributeResponse:
        """
        Retrieves a [custom attribute](entity:CustomAttribute) associated with an order.

        You can use the `with_definition` query parameter to also retrieve the custom attribute definition
        in the same call.

        To retrieve a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        order_id : str
            The ID of the target [order](entity:Order).

        custom_attribute_key : str
            The key of the custom attribute to retrieve.  This key must match the key of an
            existing custom attribute definition.

        version : typing.Optional[int]
            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include this optional field and specify the current version of the custom attribute.

        with_definition : typing.Optional[bool]
            Indicates whether to return the [custom attribute definition](entity:CustomAttributeDefinition) in the `definition` field of each
            custom attribute. Set this parameter to `true` to get the name and description of each custom attribute,
            information about the data type, or other definition details. The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveOrderCustomAttributeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.orders.custom_attributes.get(
                order_id="order_id",
                custom_attribute_key="custom_attribute_key",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(
            order_id,
            custom_attribute_key,
            version=version,
            with_definition=with_definition,
            request_options=request_options,
        )
        return response.data

    async def upsert(
        self,
        order_id: str,
        custom_attribute_key: str,
        *,
        custom_attribute: CustomAttributeParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpsertOrderCustomAttributeResponse:
        """
        Creates or updates a [custom attribute](entity:CustomAttribute) for an order.

        Use this endpoint to set the value of a custom attribute for a specific order.
        A custom attribute is based on a custom attribute definition in a Square seller account. (To create a
        custom attribute definition, use the [CreateOrderCustomAttributeDefinition](api-endpoint:OrderCustomAttributes-CreateOrderCustomAttributeDefinition) endpoint.)

        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        order_id : str
            The ID of the target [order](entity:Order).

        custom_attribute_key : str
            The key of the custom attribute to create or update.  This key must match the key
            of an existing custom attribute definition.

        custom_attribute : CustomAttributeParams
            The custom attribute to create or update, with the following fields:

            - `value`. This value must conform to the `schema` specified by the definition.
            For more information, see [Value data types](https://developer.squareup.com/docs/customer-custom-attributes-api/custom-attributes#value-data-types).

            - `version`. To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include this optional field and specify the current version of the custom attribute.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency.
            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpsertOrderCustomAttributeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.orders.custom_attributes.upsert(
                order_id="order_id",
                custom_attribute_key="custom_attribute_key",
                custom_attribute={"key": "table-number", "value": "42", "version": 1},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.upsert(
            order_id,
            custom_attribute_key,
            custom_attribute=custom_attribute,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    async def delete(
        self, order_id: str, custom_attribute_key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteOrderCustomAttributeResponse:
        """
        Deletes a [custom attribute](entity:CustomAttribute) associated with a customer profile.

        To delete a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        order_id : str
            The ID of the target [order](entity:Order).

        custom_attribute_key : str
            The key of the custom attribute to delete.  This key must match the key of an
            existing custom attribute definition.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteOrderCustomAttributeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.orders.custom_attributes.delete(
                order_id="order_id",
                custom_attribute_key="custom_attribute_key",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(order_id, custom_attribute_key, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .scheduled_shift_filter import ScheduledShiftFilter
import pydantic
from .scheduled_shift_sort import ScheduledShiftSort
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ScheduledShiftQuery(UncheckedBaseModel):
    """
    Represents filter and sort criteria for the `query` field in a
    [SearchScheduledShifts](api-endpoint:Labor-SearchScheduledShifts) request.
    """

    filter: typing.Optional[ScheduledShiftFilter] = pydantic.Field(default=None)
    """
    Filtering options for the query.
    """

    sort: typing.Optional[ScheduledShiftSort] = pydantic.Field(default=None)
    """
    Sorting options for the query.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class ReceiptOptionsParams(typing_extensions.TypedDict):
    """
    Describes receipt action fields.
    """

    payment_id: str
    """
    The reference to the Square payment ID for the receipt.
    """

    print_only: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Instructs the device to print the receipt without displaying the receipt selection screen.
    Requires `printer_enabled` set to true.
    Defaults to false.
    """

    is_duplicate: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Identify the receipt as a reprint rather than an original receipt.
    Defaults to false.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchOrdersCustomerFilter(UncheckedBaseModel):
    """
    A filter based on the order `customer_id` and any tender `customer_id`
    associated with the order. It does not filter based on the
    [FulfillmentRecipient](entity:FulfillmentRecipient) `customer_id`.
    """

    customer_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of customer IDs to filter by.
    
    Max: 10 customer ids.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .merchant import MerchantParams


class GetMerchantResponseParams(typing_extensions.TypedDict):
    """
    The response object returned by the [RetrieveMerchant](api-endpoint:Merchants-RetrieveMerchant) endpoint.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information on errors encountered during the request.
    """

    merchant: typing_extensions.NotRequired[MerchantParams]
    """
    The requested `Merchant` object.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .payment_link import PaymentLinkParams


class GetPaymentLinkResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    payment_link: typing_extensions.NotRequired[PaymentLinkParams]
    """
    The payment link that is retrieved.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.customer_sort_field import CustomerSortField
from ..types.sort_order import SortOrder


class CustomerSortParams(typing_extensions.TypedDict):
    """
    Represents the sorting criteria in a [search query](entity:CustomerQuery) that defines how to sort
    customer profiles returned in [SearchCustomers](api-endpoint:Customers-SearchCustomers) results.
    """

    field: typing_extensions.NotRequired[CustomerSortField]
    """
    Indicates the fields to use as the sort key, which is either the default set of fields or `created_at`.
    
    The default value is `DEFAULT`.
    See [CustomerSortField](#type-customersortfield) for possible values
    """

    order: typing_extensions.NotRequired[SortOrder]
    """
    Indicates the order in which results should be sorted based on the
    sort field value. Strings use standard alphabetic comparison
    to determine order. Strings representing numbers are sorted as strings.
    
    The default value is `ASC`.
    See [SortOrder](#type-sortorder) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.create_mobile_authorization_code_response import CreateMobileAuthorizationCodeResponse
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawMobileClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def authorization_code(
        self, *, location_id: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CreateMobileAuthorizationCodeResponse]:
        """
        __Note:__ This endpoint is used by the deprecated Reader SDK.
        Developers should update their integration to use the [Mobile Payments SDK](https://developer.squareup.com/docs/mobile-payments-sdk), which includes its own authorization methods.

        Generates code to authorize a mobile application to connect to a Square card reader.

        Authorization codes are one-time-use codes and expire 60 minutes after being issued.

        The `Authorization` header you provide to this endpoint must have the following format:

        ```
        Authorization: Bearer ACCESS_TOKEN
        ```

        Replace `ACCESS_TOKEN` with a
        [valid production authorization credential](https://developer.squareup.com/docs/build-basics/access-tokens).

        Parameters
        ----------
        location_id : typing.Optional[str]
            The Square location ID that the authorization code should be tied to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateMobileAuthorizationCodeResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "mobile/authorization-code",
            method="POST",
            json={
                "location_id": location_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateMobileAuthorizationCodeResponse,
                    construct_type(
                        type_=CreateMobileAuthorizationCodeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawMobileClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def authorization_code(
        self, *, location_id: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CreateMobileAuthorizationCodeResponse]:
        """
        __Note:__ This endpoint is used by the deprecated Reader SDK.
        Developers should update their integration to use the [Mobile Payments SDK](https://developer.squareup.com/docs/mobile-payments-sdk), which includes its own authorization methods.

        Generates code to authorize a mobile application to connect to a Square card reader.

        Authorization codes are one-time-use codes and expire 60 minutes after being issued.

        The `Authorization` header you provide to this endpoint must have the following format:

        ```
        Authorization: Bearer ACCESS_TOKEN
        ```

        Replace `ACCESS_TOKEN` with a
        [valid production authorization credential](https://developer.squareup.com/docs/build-basics/access-tokens).

        Parameters
        ----------
        location_id : typing.Optional[str]
            The Square location ID that the authorization code should be tied to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateMobileAuthorizationCodeResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "mobile/authorization-code",
            method="POST",
            json={
                "location_id": location_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateMobileAuthorizationCodeResponse,
                    construct_type(
                        type_=CreateMobileAuthorizationCodeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.loyalty_program_reward_definition_scope import LoyaltyProgramRewardDefinitionScope
from ..types.loyalty_program_reward_definition_type import LoyaltyProgramRewardDefinitionType
import typing_extensions
import typing
from .money import MoneyParams


class LoyaltyProgramRewardDefinitionParams(typing_extensions.TypedDict):
    """
    Provides details about the reward tier discount. DEPRECATED at version 2020-12-16. Discount details
    are now defined using a catalog pricing rule and other catalog objects. For more information, see
    [Getting discount details for a reward tier](https://developer.squareup.com/docs/loyalty-api/loyalty-rewards#get-discount-details).
    """

    scope: LoyaltyProgramRewardDefinitionScope
    """
    Indicates the scope of the reward tier. DEPRECATED at version 2020-12-16. You can find this information in the
    `product_set_data` field of the `PRODUCT_SET` catalog object referenced by the pricing rule. For `ORDER` scopes,
    `all_products` is true. For `ITEM_VARIATION` or `CATEGORY` scopes, `product_ids_any` is a list of
    catalog object IDs of the given type.
    See [LoyaltyProgramRewardDefinitionScope](#type-loyaltyprogramrewarddefinitionscope) for possible values
    """

    discount_type: LoyaltyProgramRewardDefinitionType
    """
    The type of discount the reward tier offers. DEPRECATED at version 2020-12-16. You can find this information
    in the `discount_data.discount_type` field of the `DISCOUNT` catalog object referenced by the pricing rule.
    See [LoyaltyProgramRewardDefinitionType](#type-loyaltyprogramrewarddefinitiontype) for possible values
    """

    percentage_discount: typing_extensions.NotRequired[str]
    """
    The fixed percentage of the discount. Present if `discount_type` is `FIXED_PERCENTAGE`.
    For example, a 7.25% off discount will be represented as "7.25". DEPRECATED at version 2020-12-16. You can find this
    information in the `discount_data.percentage` field of the `DISCOUNT` catalog object referenced by the pricing rule.
    """

    catalog_object_ids: typing_extensions.NotRequired[typing.Sequence[str]]
    """
    The list of catalog objects to which this reward can be applied. They are either all item-variation ids or category ids, depending on the `type` field.
    DEPRECATED at version 2020-12-16. You can find this information in the `product_set_data.product_ids_any` field
    of the `PRODUCT_SET` catalog object referenced by the pricing rule.
    """

    fixed_discount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of the discount. Present if `discount_type` is `FIXED_AMOUNT`. For example, $5 off.
    DEPRECATED at version 2020-12-16. You can find this information in the `discount_data.amount_money` field of the 
    `DISCOUNT` catalog object referenced by the pricing rule.
    """

    max_discount_money: typing_extensions.NotRequired[MoneyParams]
    """
    When `discount_type` is `FIXED_PERCENTAGE`, the maximum discount amount that can be applied.
    DEPRECATED at version 2020-12-16. You can find this information in the `discount_data.maximum_amount_money` field
    of the `DISCOUNT` catalog object referenced by the the pricing rule.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

ListPaymentsRequestSortField = typing.Union[
    typing.Literal["CREATED_AT", "OFFLINE_CREATED_AT", "UPDATED_AT"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class AchDetails(UncheckedBaseModel):
    """
    ACH-specific details about `BANK_ACCOUNT` type payments with the `transfer_type` of `ACH`.
    """

    routing_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The routing number for the bank account.
    """

    account_number_suffix: typing.Optional[str] = pydantic.Field(default=None)
    """
    The last few digits of the bank account number.
    """

    account_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of the bank account performing the transfer. The account type can be `CHECKING`,
    `SAVINGS`, or `UNKNOWN`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

InvoiceRequestMethod = typing.Union[
    typing.Literal[
        "EMAIL",
        "CHARGE_CARD_ON_FILE",
        "SHARE_MANUALLY",
        "CHARGE_BANK_ON_FILE",
        "SMS",
        "SMS_CHARGE_CARD_ON_FILE",
        "SMS_CHARGE_BANK_ON_FILE",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawWebhooksClient
from .event_types.client import EventTypesClient
from .subscriptions.client import SubscriptionsClient
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawWebhooksClient
from .event_types.client import AsyncEventTypesClient
from .subscriptions.client import AsyncSubscriptionsClient


class WebhooksClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawWebhooksClient(client_wrapper=client_wrapper)
        self.event_types = EventTypesClient(client_wrapper=client_wrapper)

        self.subscriptions = SubscriptionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawWebhooksClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawWebhooksClient
        """
        return self._raw_client


class AsyncWebhooksClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawWebhooksClient(client_wrapper=client_wrapper)
        self.event_types = AsyncEventTypesClient(client_wrapper=client_wrapper)

        self.subscriptions = AsyncSubscriptionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawWebhooksClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawWebhooksClient
        """
        return self._raw_client



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .customer_group import CustomerGroupParams


class GetCustomerGroupResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of
    a request to the [RetrieveCustomerGroup](api-endpoint:CustomerGroups-RetrieveCustomerGroup) endpoint.

    Either `errors` or `group` is present in a given response (never both).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    group: typing_extensions.NotRequired[CustomerGroupParams]
    """
    The retrieved customer group.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .sort_order import SortOrder
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TerminalActionQuerySort(UncheckedBaseModel):
    sort_order: typing.Optional[SortOrder] = pydantic.Field(default=None)
    """
    The order in which results are listed.
    - `ASC` - Oldest to newest.
    - `DESC` - Newest to oldest (default).
    See [SortOrder](#type-sortorder) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .shift import ShiftParams
import typing
from .error import ErrorParams


class CreateShiftResponseParams(typing_extensions.TypedDict):
    """
    The response to a request to create a `Shift`. The response contains
    the created `Shift` object and might contain a set of `Error` objects if
    the request resulted in errors.
    """

    shift: typing_extensions.NotRequired[ShiftParams]
    """
    The `Shift` that was created on the request.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams


class DeleteCustomerCustomAttributeResponseParams(typing_extensions.TypedDict):
    """
    Represents a [DeleteCustomerCustomAttribute](api-endpoint:CustomerCustomAttributes-DeleteCustomerCustomAttribute) response.
    Either an empty object `{}` (for a successful deletion) or `errors` is present in the response.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .job import JobParams
from .error import ErrorParams


class ListJobsResponseParams(typing_extensions.TypedDict):
    """
    Represents a [ListJobs](api-endpoint:Team-ListJobs) response. Either `jobs` or `errors`
    is present in the response. If additional results are available, the `cursor` field is also present.
    """

    jobs: typing_extensions.NotRequired[typing.Sequence[JobParams]]
    """
    The retrieved jobs. A single paged response contains up to 100 jobs.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    An opaque cursor used to retrieve the next page of results. This field is present only
    if the request succeeded and additional results are available. For more information, see
    [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    The errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .search_events_filter import SearchEventsFilter
import pydantic
from .search_events_sort import SearchEventsSort
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchEventsQuery(UncheckedBaseModel):
    """
    Contains query criteria for the search.
    """

    filter: typing.Optional[SearchEventsFilter] = pydantic.Field(default=None)
    """
    Criteria to filter events by.
    """

    sort: typing.Optional[SearchEventsSort] = pydantic.Field(default=None)
    """
    Criteria to sort events by.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .gift_card_activity_type import GiftCardActivityType
from .money import Money
from .gift_card_activity_load import GiftCardActivityLoad
from .gift_card_activity_activate import GiftCardActivityActivate
from .gift_card_activity_redeem import GiftCardActivityRedeem
from .gift_card_activity_clear_balance import GiftCardActivityClearBalance
from .gift_card_activity_deactivate import GiftCardActivityDeactivate
from .gift_card_activity_adjust_increment import GiftCardActivityAdjustIncrement
from .gift_card_activity_adjust_decrement import GiftCardActivityAdjustDecrement
from .gift_card_activity_refund import GiftCardActivityRefund
from .gift_card_activity_unlinked_activity_refund import GiftCardActivityUnlinkedActivityRefund
from .gift_card_activity_import import GiftCardActivityImport
from .gift_card_activity_block import GiftCardActivityBlock
from .gift_card_activity_unblock import GiftCardActivityUnblock
from .gift_card_activity_import_reversal import GiftCardActivityImportReversal
from .gift_card_activity_transfer_balance_to import GiftCardActivityTransferBalanceTo
from .gift_card_activity_transfer_balance_from import GiftCardActivityTransferBalanceFrom
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class GiftCardActivity(UncheckedBaseModel):
    """
    Represents an action performed on a [gift card](entity:GiftCard) that affects its state or balance.
    A gift card activity contains information about a specific activity type. For example, a `REDEEM` activity
    includes a `redeem_activity_details` field that contains information about the redemption.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-assigned ID of the gift card activity.
    """

    type: GiftCardActivityType = pydantic.Field()
    """
    The type of gift card activity.
    See [Type](#type-type) for possible values
    """

    location_id: str = pydantic.Field()
    """
    The ID of the [business location](entity:Location) where the activity occurred.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the gift card activity was created, in RFC 3339 format.
    """

    gift_card_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The gift card ID. When creating a gift card activity, `gift_card_id` is not required if 
    `gift_card_gan` is specified.
    """

    gift_card_gan: typing.Optional[str] = pydantic.Field(default=None)
    """
    The gift card account number (GAN). When creating a gift card activity, `gift_card_gan` 
    is not required if `gift_card_id` is specified.
    """

    gift_card_balance_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The final balance on the gift card after the action is completed.
    """

    load_activity_details: typing.Optional[GiftCardActivityLoad] = pydantic.Field(default=None)
    """
    Additional details about a `LOAD` activity, which is used to reload money onto a gift card.
    """

    activate_activity_details: typing.Optional[GiftCardActivityActivate] = pydantic.Field(default=None)
    """
    Additional details about an `ACTIVATE` activity, which is used to activate a gift card with 
    an initial balance.
    """

    redeem_activity_details: typing.Optional[GiftCardActivityRedeem] = pydantic.Field(default=None)
    """
    Additional details about a `REDEEM` activity, which is used to redeem a gift card for a purchase.
    
    For applications that process payments using the Square Payments API, Square creates a `REDEEM` activity that 
    updates the gift card balance after the corresponding [CreatePayment](api-endpoint:Payments-CreatePayment) 
    request is completed. Applications that use a custom payment processing system must call 
    [CreateGiftCardActivity](api-endpoint:GiftCardActivities-CreateGiftCardActivity) to create the `REDEEM` activity.
    """

    clear_balance_activity_details: typing.Optional[GiftCardActivityClearBalance] = pydantic.Field(default=None)
    """
    Additional details about a `CLEAR_BALANCE` activity, which is used to set the balance of a gift card to zero.
    """

    deactivate_activity_details: typing.Optional[GiftCardActivityDeactivate] = pydantic.Field(default=None)
    """
    Additional details about a `DEACTIVATE` activity, which is used to deactivate a gift card.
    """

    adjust_increment_activity_details: typing.Optional[GiftCardActivityAdjustIncrement] = pydantic.Field(default=None)
    """
    Additional details about an `ADJUST_INCREMENT` activity, which is used to add money to a gift card 
    outside of a typical `ACTIVATE`, `LOAD`, or `REFUND` activity flow.
    """

    adjust_decrement_activity_details: typing.Optional[GiftCardActivityAdjustDecrement] = pydantic.Field(default=None)
    """
    Additional details about an `ADJUST_DECREMENT` activity, which is used to deduct money from a gift 
    card outside of a typical `REDEEM` activity flow.
    """

    refund_activity_details: typing.Optional[GiftCardActivityRefund] = pydantic.Field(default=None)
    """
    Additional details about a `REFUND` activity, which is used to add money to a gift card when 
    refunding a payment.
    
    For applications that refund payments to a gift card using the Square Refunds API, Square automatically
    creates a `REFUND` activity that updates the gift card balance after a [RefundPayment](api-endpoint:Refunds-RefundPayment)
    request is completed. Applications that use a custom processing system must call
    [CreateGiftCardActivity](api-endpoint:GiftCardActivities-CreateGiftCardActivity) to create the `REFUND` activity.
    """

    unlinked_activity_refund_activity_details: typing.Optional[GiftCardActivityUnlinkedActivityRefund] = pydantic.Field(
        default=None
    )
    """
    Additional details about an `UNLINKED_ACTIVITY_REFUND` activity. This activity is used to add money 
    to a gift card when refunding a payment that was processed using a custom payment processing system
    and not linked to the gift card.
    """

    import_activity_details: typing.Optional[GiftCardActivityImport] = pydantic.Field(default=None)
    """
    Additional details about an `IMPORT` activity, which Square uses to import a third-party 
    gift card with a balance.
    """

    block_activity_details: typing.Optional[GiftCardActivityBlock] = pydantic.Field(default=None)
    """
    Additional details about a `BLOCK` activity, which Square uses to temporarily block a gift card.
    """

    unblock_activity_details: typing.Optional[GiftCardActivityUnblock] = pydantic.Field(default=None)
    """
    Additional details about an `UNBLOCK` activity, which Square uses to unblock a gift card.
    """

    import_reversal_activity_details: typing.Optional[GiftCardActivityImportReversal] = pydantic.Field(default=None)
    """
    Additional details about an `IMPORT_REVERSAL` activity, which Square uses to reverse the 
    import of a third-party gift card.
    """

    transfer_balance_to_activity_details: typing.Optional[GiftCardActivityTransferBalanceTo] = pydantic.Field(
        default=None
    )
    """
    Additional details about a `TRANSFER_BALANCE_TO` activity, which Square uses to add money to
    a gift card as the result of a transfer from another gift card.
    """

    transfer_balance_from_activity_details: typing.Optional[GiftCardActivityTransferBalanceFrom] = pydantic.Field(
        default=None
    )
    """
    Additional details about a `TRANSFER_BALANCE_FROM` activity, which Square uses to deduct money from
    a gift as the result of a transfer to another gift card.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CheckoutLocationSettingsTipping(UncheckedBaseModel):
    percentages: typing.Optional[typing.List[int]] = pydantic.Field(default=None)
    """
    Set three custom percentage amounts that buyers can select at checkout. If Smart Tip is enabled, this only applies to transactions totaling $10 or more.
    """

    smart_tipping_enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables Smart Tip Amounts. If Smart Tip Amounts is enabled, tipping works as follows:
    If a transaction is less than $10, the available tipping options include No Tip, $1, $2, or $3.
    If a transaction is $10 or more, the available tipping options include No Tip, 15%, 20%, or 25%. 
    You can set custom percentage amounts with the `percentages` field.
    """

    default_percent: typing.Optional[int] = pydantic.Field(default=None)
    """
    Set the pre-selected percentage amounts that appear at checkout. If Smart Tip is enabled, this only applies to transactions totaling $10 or more.
    """

    smart_tips: typing.Optional[typing.List[Money]] = pydantic.Field(default=None)
    """
    Show the Smart Tip Amounts for this location.
    """

    default_smart_tip: typing.Optional[Money] = pydantic.Field(default=None)
    """
    Set the pre-selected whole amount that appears at checkout when Smart Tip is enabled and the transaction amount is less than $10.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .money import Money
import pydantic
import typing
from .loyalty_program_accrual_rule_tax_mode import LoyaltyProgramAccrualRuleTaxMode
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LoyaltyProgramAccrualRuleSpendData(UncheckedBaseModel):
    """
    Represents additional data for rules with the `SPEND` accrual type.
    """

    amount_money: Money = pydantic.Field()
    """
    The amount that buyers must spend to earn points. 
    For example, given an "Earn 1 point for every $10 spent" accrual rule, a buyer who spends $105 earns 10 points.
    """

    excluded_category_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The IDs of any `CATEGORY` catalog objects that are excluded from points accrual.
    
    You can use the [BatchRetrieveCatalogObjects](api-endpoint:Catalog-BatchRetrieveCatalogObjects)
    endpoint to retrieve information about the excluded categories.
    """

    excluded_item_variation_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The IDs of any `ITEM_VARIATION` catalog objects that are excluded from points accrual.
    
    You can use the [BatchRetrieveCatalogObjects](api-endpoint:Catalog-BatchRetrieveCatalogObjects)
    endpoint to retrieve information about the excluded item variations.
    """

    tax_mode: LoyaltyProgramAccrualRuleTaxMode = pydantic.Field()
    """
    Indicates how taxes should be treated when calculating the purchase amount used for points accrual.
    See [LoyaltyProgramAccrualRuleTaxMode](#type-loyaltyprogramaccrualruletaxmode) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

SubscriptionCadence = typing.Union[
    typing.Literal[
        "DAILY",
        "WEEKLY",
        "EVERY_TWO_WEEKS",
        "THIRTY_DAYS",
        "SIXTY_DAYS",
        "NINETY_DAYS",
        "MONTHLY",
        "EVERY_TWO_MONTHS",
        "QUARTERLY",
        "EVERY_FOUR_MONTHS",
        "EVERY_SIX_MONTHS",
        "ANNUAL",
        "EVERY_TWO_YEARS",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .filter_value import FilterValue
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SegmentFilter(UncheckedBaseModel):
    """
    A query filter to search for buyer-accessible appointment segments by.
    """

    service_variation_id: str = pydantic.Field()
    """
    The ID of the [CatalogItemVariation](entity:CatalogItemVariation) object representing the service booked in this segment.
    """

    team_member_id_filter: typing.Optional[FilterValue] = pydantic.Field(default=None)
    """
    A query filter to search for buyer-accessible appointment segments with service-providing team members matching the specified list of team member IDs.  Supported query expressions are
    - `ANY`: return the appointment segments with team members whose IDs match any member in this list.
    - `NONE`: return the appointment segments with team members whose IDs are not in this list.
    - `ALL`: not supported.
    
    When no expression is specified, any service-providing team member is eligible to fulfill the Booking.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.catalog_discount_type import CatalogDiscountType
from .money import MoneyParams
from ..types.catalog_discount_modify_tax_basis import CatalogDiscountModifyTaxBasis


class CatalogDiscountParams(typing_extensions.TypedDict):
    """
    A discount applicable to items.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The discount name. This is a searchable attribute for use in applicable query filters, and its value length is of Unicode code points.
    """

    discount_type: typing_extensions.NotRequired[CatalogDiscountType]
    """
    Indicates whether the discount is a fixed amount or percentage, or entered at the time of sale.
    See [CatalogDiscountType](#type-catalogdiscounttype) for possible values
    """

    percentage: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The percentage of the discount as a string representation of a decimal number, using a `.` as the decimal
    separator and without a `%` sign. A value of `7.5` corresponds to `7.5%`. Specify a percentage of `0` if `discount_type`
    is `VARIABLE_PERCENTAGE`.
    
    Do not use this field for amount-based or variable discounts.
    """

    amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of the discount. Specify an amount of `0` if `discount_type` is `VARIABLE_AMOUNT`.
    
    Do not use this field for percentage-based or variable discounts.
    """

    pin_required: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether a mobile staff member needs to enter their PIN to apply the
    discount to a payment in the Square Point of Sale app.
    """

    label_color: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The color of the discount display label in the Square Point of Sale app. This must be a valid hex color code.
    """

    modify_tax_basis: typing_extensions.NotRequired[CatalogDiscountModifyTaxBasis]
    """
    Indicates whether this discount should reduce the price used to calculate tax.
    
    Most discounts should use `MODIFY_TAX_BASIS`. However, in some circumstances taxes must
    be calculated based on an item's price, ignoring a particular discount. For example,
    in many US jurisdictions, a manufacturer coupon or instant rebate reduces the price a
    customer pays but does not reduce the sale price used to calculate how much sales tax is
    due. In this case, the discount representing that manufacturer coupon should have
    `DO_NOT_MODIFY_TAX_BASIS` for this field.
    
    If you are unsure whether you need to use this field, consult your tax professional.
    See [CatalogDiscountModifyTaxBasis](#type-catalogdiscountmodifytaxbasis) for possible values
    """

    maximum_amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    For a percentage discount, the maximum absolute value of the discount. For example, if a
    50% discount has a `maximum_amount_money` of $20, a $100 purchase will yield a $20 discount,
    not a $50 discount.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .money import MoneyParams
import typing_extensions


class DestinationDetailsCashRefundDetailsParams(typing_extensions.TypedDict):
    """
    Stores details about a cash refund. Contains only non-confidential information.
    """

    seller_supplied_money: MoneyParams
    """
    The amount and currency of the money supplied by the seller.
    """

    change_back_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of change due back to the seller.
    This read-only field is calculated
    from the `amount_money` and `seller_supplied_money` fields.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

BusinessBookingProfileCustomerTimezoneChoice = typing.Union[
    typing.Literal["BUSINESS_LOCATION_TIMEZONE", "CUSTOMER_CHOICE"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.business_appointment_settings_booking_location_type import BusinessAppointmentSettingsBookingLocationType
from ..types.business_appointment_settings_alignment_time import BusinessAppointmentSettingsAlignmentTime
from ..types.business_appointment_settings_max_appointments_per_day_limit_type import (
    BusinessAppointmentSettingsMaxAppointmentsPerDayLimitType,
)
from .money import MoneyParams
from ..types.business_appointment_settings_cancellation_policy import BusinessAppointmentSettingsCancellationPolicy


class BusinessAppointmentSettingsParams(typing_extensions.TypedDict):
    """
    The service appointment settings, including where and how the service is provided.
    """

    location_types: typing_extensions.NotRequired[
        typing.Optional[typing.Sequence[BusinessAppointmentSettingsBookingLocationType]]
    ]
    """
    Types of the location allowed for bookings.
    See [BusinessAppointmentSettingsBookingLocationType](#type-businessappointmentsettingsbookinglocationtype) for possible values
    """

    alignment_time: typing_extensions.NotRequired[BusinessAppointmentSettingsAlignmentTime]
    """
    The time unit of the service duration for bookings.
    See [BusinessAppointmentSettingsAlignmentTime](#type-businessappointmentsettingsalignmenttime) for possible values
    """

    min_booking_lead_time_seconds: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The minimum lead time in seconds before a service can be booked. A booking must be created at least this amount of time before its starting time.
    """

    max_booking_lead_time_seconds: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The maximum lead time in seconds before a service can be booked. A booking must be created at most this amount of time before its starting time.
    """

    any_team_member_booking_enabled: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether a customer can choose from all available time slots and have a staff member assigned
    automatically (`true`) or not (`false`).
    """

    multiple_service_booking_enabled: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether a customer can book multiple services in a single online booking.
    """

    max_appointments_per_day_limit_type: typing_extensions.NotRequired[
        BusinessAppointmentSettingsMaxAppointmentsPerDayLimitType
    ]
    """
    Indicates whether the daily appointment limit applies to team members or to
    business locations.
    See [BusinessAppointmentSettingsMaxAppointmentsPerDayLimitType](#type-businessappointmentsettingsmaxappointmentsperdaylimittype) for possible values
    """

    max_appointments_per_day_limit: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The maximum number of daily appointments per team member or per location.
    """

    cancellation_window_seconds: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The cut-off time in seconds for allowing clients to cancel or reschedule an appointment.
    """

    cancellation_fee_money: typing_extensions.NotRequired[MoneyParams]
    """
    The flat-fee amount charged for a no-show booking.
    """

    cancellation_policy: typing_extensions.NotRequired[BusinessAppointmentSettingsCancellationPolicy]
    """
    The cancellation policy adopted by the seller.
    See [BusinessAppointmentSettingsCancellationPolicy](#type-businessappointmentsettingscancellationpolicy) for possible values
    """

    cancellation_policy_text: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The free-form text of the seller's cancellation policy.
    """

    skip_booking_flow_staff_selection: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether customers has an assigned staff member (`true`) or can select s staff member of their choice (`false`).
    """
